/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.container.config;

import java.util.Iterator;

import org.netkernel.urii.INetKernelException;

/** Interface for a object supplying configuration state to the kernel
 * @author Tony Butterfield
 */
public interface IConfiguration
{
	/** Return a configuration value as a java.lang.String
	 * @param aKey the key of the configuration value
	 * @return the value
	 * @throws INetKernelException thrown if no value is found for the given key
	 */
	String getString(String aKey) throws INetKernelException;
	
	/** Return a configuration value as a java.lang.String
	 * @param aKey the key of the configuration value
	 * @param aDefault a default value for the case that no value if found
	 * @return the value
	 */
	String getString(String aKey, String aDefault);
	
	/** Return a configuration value as a java.lang.Integer
	 * @param aKey the key of the configuration value
	 * @return the value
	 * @throws INetKernelException thrown if no value is found for the given key
	 */
	int getInt(String aKey)  throws INetKernelException;
	
	/** Return a configuration value as a java.lang.Integer
	 * @param aKey the key of the configuration value
	 * @param aDefault a default value for the case that no value if found
	 * @return the value
	 */
	int getInt(String aKey, int aDefault);
	
	/** Return a configuration value as a java.lang.Long
	 * @param aKey the key of the configuration value
	 * @return the value
	 * @throws INetKernelException thrown if no value is found for the given key
	 */
	long getLong(String aKey)  throws INetKernelException;
	
	/** Return a configuration value as a java.lang.Long
	 * @param aKey the key of the configuration value
	 * @param aDefault a default value for the case that no value if found
	 * @return the value
	 */
	long getLong(String aKey, long aDefault);
	
	/** Return a configuration value as a java.lang.Boolean
	 * @param aKey the key of the configuration value
	 * @return the value
	 * @throws INetKernelException thrown if no value is found for the given key
	 */
	boolean getBoolean(String aKey) throws INetKernelException;
	
	/** Return a configuration value as a java.lang.Boolean
	 * @param aKey the key of the configuration value
	 * @param aDefault a default value for the case that no value if found
	 * @return the value
	 */
	boolean getBoolean(String aKey, boolean aDefault);
	
	/** Return an iterator over the configuration key Strings.
	 * @return the iterator
	 */
	Iterator getKeys();
	
	/** Tests if the given key is known
	 * @return boolean true if known
	 */
	boolean containsKey(String key);
	
}
