/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.scheduler;

import java.io.IOException;
import java.io.Writer;

import org.netkernel.container.IKernel;
import org.netkernel.request.IRequest;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.urii.impl.NetKernelException;
import org.netkernel.util.Utils;
import org.netkernel.util.XMLUtils;

/** Exception when a request resolution fails it contains a request debug trace
 * @author  Tony Butterfield
 */
public class ResolutionFailureException extends NetKernelException
{
	private final TraceDebugCapture mDebug;
	private final IRequest mRequest;
	
	/** Creates a new instance of ResolutionFailureException */
	public ResolutionFailureException(IKernel aKernel, IRequest aRequest, TraceDebugCapture aDebug)
	{	super(aKernel.getLogger().format("EX_RESOLUTION_FAILURE"),RequestFactory.toShortString(aRequest),null);
		mDebug=aDebug;
		mRequest=aRequest;
	}
	
	public IRequest getRequest()
	{	return mRequest;
	}
	
	public TraceDebugCapture getTraceDebugCapture()
	{	return mDebug;
	}
	
	public void writeAdditionalFieldsAsXML(Writer aWriter, int aIndent) throws IOException
	{	XMLUtils.writeOpenTag(aWriter, "trace", aIndent);
		if (aIndent>=0) aIndent++;
		TraceDebugCapture tdc = getTraceDebugCapture();
		Utils.writeTraceDebug(aWriter,tdc,aIndent);
		XMLUtils.writeCloseTag(aWriter, "trace", aIndent-1);		
	}	
}