/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.util;

import java.util.concurrent.atomic.AtomicReference;

import org.netkernel.request.IResponse;

/** JDK1.5+ implementation 
 *  a threadsafe implementation that holds a response whilst it is waiting for it otherwise rejects it
 *  - this is used to make asynchronous request-response pairs synchronous if response comes 
 *  back before request has completed  
 * @author Tony Butterfield
 *
 */
public class AsynchResponseJoin
{	private AtomicReference<Object> mReference = new AtomicReference<Object>();

	public static final Integer HANDLED=new Integer(0);
	public static final Integer HANDLE_ASYNC=new Integer(1);
	public static final Integer HANDLE_ASYNC_DOUBLE=new Integer(2);
	
	public void startCapture()
	{	mReference.set(null);
	}
	
	public IResponse stopCapture()
	{	IResponse result;
		Object response = mReference.getAndSet(HANDLED);
		if (response instanceof IResponse)
		{	result=(IResponse)response;
		}
		else
		{	result=null;
		}
		return result;
	}
	
	/** Return an instruction on how to handle response,
	 * HANDLED means that it's been captured to be handled
	 * HANDLE_ASYNC means that it must be handled now asynchronously
	 * HANDLE_ASYNC_DOUBLE means that we have already had a response and this is a duplicate
	 * @param aResponse
	 * @return
	 */
	public Integer acceptResponse(IResponse aResponse)
	{	Integer result;
		if (mReference.compareAndSet(null,aResponse))
		{	result=HANDLED;
		}
		else
		{	if (mReference.compareAndSet(HANDLED,HANDLE_ASYNC))
			{	result=HANDLE_ASYNC;
			}
			else
			{	result=HANDLE_ASYNC_DOUBLE;
			}
		}
		return result;
	}
	
	public boolean isAccepting()
	{	return mReference.get()==null;
	}
}