/******************************************************************************
  (c) Copyright 2002-2008, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      
 *****************************************************************************/
package org.netkernel.util;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Map;
import java.util.WeakHashMap;

import org.netkernel.container.IKernel;
import org.netkernel.container.IMessages;
import org.netkernel.request.*;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.ResponseImpl;
import org.netkernel.scheduler.*;
import org.netkernel.urii.*;
import org.netkernel.urii.impl.NetKernelError;
import org.netkernel.urii.impl.NetKernelException;

/** Set of static utility methods
 * @author  tab
 */
public class Utils
{
	
	/** Construct a INetKernelException with optional id, message and requestid. If formats id and message message by looking up in
	 * the IMessages object. An optional array of up to four parameters may be supplied for the message formatting.
	 * @param aId An optional id for the exception
	 * @param aMessage An optional message for the exception
	 * @param aRequestId An optional requestid for the exception
	 * @param aMessages  the IMessages implementation to supply formatting of id and message
	 * @param aParams an optional array of parameters supplied to the message formatting
	 * @return the created exception
	 */	
	public static INetKernelException createFormattedException(String aId, String aMessage, IMessages aMessages, Object... aParams)
	{	return createFormattedException(aId,aMessage,aMessages,null,aParams);
	}
	
	public static INetKernelException createFormattedException(String aId, String aMessage, IMessages aMessages, Throwable aCause, Object... aParams)
	{	String id=(aId!=null)?aMessages.format(aId):null;
		String message=null;
		if (aMessage!=null)
		{	message=aMessages.format(aMessage,aParams);
		}	
		return new NetKernelException(id,message,aCause);
	}
	
	public static IResponse generateExceptionResponse(IRequest aRequest, IKernel aKernel, String aId)
	{	String id = aKernel.getLogger().format(aId);
		INetKernelException ex = new NetKernelException(id,null,null);
		return generateExceptionResponse(aRequest,ex);
	}
	
	
	public static IResponse generateExceptionResponse(IRequest aRequest, IKernel aKernel, String aId, Throwable aException)
	{	String id = aKernel.getLogger().format(aId);
		INetKernelException ex = new NetKernelException(id,null,aException);
		return generateExceptionResponse(aRequest,ex);
	}
	
	public static INetKernelException generateException(IKernel aKernel, String aId, Throwable aException)
	{	String id = aKernel.getLogger().format(aId);
		INetKernelException ex = new NetKernelException(id,null,aException);
		return ex;
	}
	
	public static IResponse generateExceptionResponse(IRequest aRequest, Throwable aThrowable)
	{	return new ResponseImpl(aRequest, aThrowable, MetaImpl.EXPIRED);	
	}
	
	/** Format a relative expiry time into a human readable debug String
	 * @param t the time offset from now
	 * @return a human readable string
	 */
	public static String expiryToString(long t)
	{	String result;
		if (t<0)
		{	result="expired";
		}
		else
		{	String units;
			if (t<1000)
			{	units="ms";
			}
			else if (t<60000)
			{	units="s";
				t/=1000;
			}
			else if (t<3600000)
			{	units="mins";
				t/=60000;
			}
			else if (t<86400000)
			{	units="hours";
				t/=3600000;
			}
			else
			{	units="days";
				t/=86400000;
			}
			result= Long.toString(t)+units;
		}
		return result;
	}
	
	/** Returns back a stringified error, for NK exception it's just a toString() but for
	 * others it will dump the stacktrace
	 * @param aThrowable
	 * @return
	 */
	public static String throwableToString(Throwable aThrowable)
	{	StringWriter sw=new StringWriter(512);
		try
		{	writeThrowableAsXML(aThrowable,sw,0);
		}
		catch (IOException e)
		{ ; }
		return sw.toString();
	}
	
	/** A toString representation of an array [<item1>, <item2>]
	 * @param aArray array to convert to string
	 * @param aExpectedLength the expected length of output
	 * @return the generated string
	 */
	public static String arrayToString(Object[] aArray, int aExpectedLength)
	{	StringBuilder sb=new StringBuilder(aExpectedLength);
		sb.append('[');
		for (int i=0; i<aArray.length; i++)
		{	if (i>0)
			{	sb.append(", ");
			}
			Object o=aArray[i];
			String s=(o==null)?"null":o.toString();
			sb.append(s);
		}
		sb.append(']');
		return sb.toString();
	}
	
	public static void writeThrowableAsXML(Throwable aThrowable, Writer aWriter, int aD) throws IOException
	{	Throwable t=aThrowable;
		XMLUtils.writeOpenTag(aWriter, "ex", aD);
		int d=(aD>=0)?aD+1:aD;
		while (t!=null)
		{	int skip=writeFrame(t,aWriter,d);
			t = t.getCause();
			for (int i=0; i<skip; i++)
			{	t=t.getCause();
			}
		}
		XMLUtils.writeCloseTag(aWriter, "ex", aD);
	}
	
	private static int writeFrame(Throwable aThrowable, Writer aWriter, int d) throws IOException
	{
		int skip=0;
		XMLUtils.writeOpenTag(aWriter, "ex", d);
		d++;
		if (aThrowable instanceof INetKernelThrowable)
		{	INetKernelThrowable nkt = (INetKernelThrowable)aThrowable;
		
			XMLUtils.writeElement(aWriter, "id", nkt.getId(),d);
			if (aThrowable.getMessage()!=null)
			{	XMLUtils.writeElement(aWriter,"message",XMLUtils.escape(aThrowable.getMessage()),d);
			}
			if (nkt.showNativeStack())
			{	writeNativeStack((Throwable)aThrowable,aWriter,d);
			}
			if (nkt instanceof NetKernelException)
			{	((NetKernelException)nkt).writeAdditionalFieldsAsXML(aWriter,d);
			}
			else if (nkt instanceof NetKernelError)
			{	((NetKernelError)nkt).writeAdditionalFieldsAsXML(aWriter,d);
			}
			
			if (aThrowable instanceof SubrequestException)
			{	skip=writeSREx( (SubrequestException)aThrowable, aWriter, d);
			}
			else if (aThrowable instanceof SubrequestError)
			{	skip=writeSREr( (SubrequestError)aThrowable, aWriter, d);
			}
		}
		else
		{	XMLUtils.writeElement(aWriter, "id", aThrowable.getClass().getName(),d);
			try
			{	String msg=aThrowable.getMessage();
				if (msg!=null)
				{	XMLUtils.writeElement(aWriter,"message",XMLUtils.escape(msg),d);
				}
			}
			catch (Exception e) {;}
			writeNativeStack(aThrowable,aWriter,d);
		}
		
		XMLUtils.writeCloseTag(aWriter, "ex", --d);
		return skip;
	}

	
	public static void writeNativeStack(Throwable aThrowable, Writer aWriter, int d) throws IOException
	{	
		XMLUtils.writeOpenTag(aWriter, "stack", d);
		if (d>=0) d++;
		StackTraceElement[] trace = aThrowable.getStackTrace();
		int traceDepth=NetKernelException.getTraceDepth();
		int i;
		boolean inKernel=false;
		for (i=0; i<traceDepth && i<trace.length; i++)
		{	StackTraceElement ste = trace[i];
			if (ste.getClassName().startsWith("org.netkernel.scheduler"))
			{	if (i==0)
				{	inKernel=true;
				}
				else if (!inKernel && i>0) break;
			}
			String message=ste.getClassName()+'.'+XMLUtils.escape(ste.getMethodName())+"() line:"+Integer.toString(ste.getLineNumber());
			XMLUtils.writeElement(aWriter, "level", message,d);
		}
		if (trace.length>i)
		{	XMLUtils.writeElement(aWriter, "level", "... "+(trace.length-i)+" more",d);
		}
		XMLUtils.writeCloseTag(aWriter, "stack", d-1);
	}

	private static int writeSREx(SubrequestException aSRE, Writer aWriter, int d) throws IOException
	{	
		ISpace ctx = aSRE.getRequestOut().getRequestScope().getSpace();
		XMLUtils.writeElement(aWriter, "space",ctx.toString(),d);
		
		IIdentifier endpointId=aSRE.getEndpointIdentifier();

		String endpointString=null;
		if (endpointId!=null && endpointId.toString()!=null )
		{	XMLUtils.writeElement(aWriter, "endpointId",endpointId.toString(),d);
		}
		
		if (endpointString==null)
		{	endpointString = aSRE.getRequestee().toString();
		}
		XMLUtils.writeElement(aWriter, "endpoint",endpointString,d);		
		
		
		int skip=0;
		Throwable t=aSRE.getCause();
		
		while (t!=null && !(t instanceof RequestFrameException || t instanceof RequestFrameError))
		{	writeFrame(t, aWriter, d);
			skip++;
			t=t.getCause();
		}
		
		return skip;
	}
	
	private static int writeSREr(SubrequestError aSRE, Writer aWriter, int d) throws IOException
	{
		ISpace ctx = aSRE.getRequestOut().getRequestScope().getSpace();
		XMLUtils.writeElement(aWriter, "space",ctx.toString(),d);
		
		IIdentifier endpointId=aSRE.getEndpointIdentifier();

		String endpointString=null;
		if (endpointId!=null && endpointId.toString()!=null )
		{	XMLUtils.writeElement(aWriter, "endpointId",endpointId.toString(),d);
		}
		
		if (endpointString==null)
		{	endpointString = aSRE.getRequestee().getClass().getName();
		}
		XMLUtils.writeElement(aWriter, "endpoint",endpointString,d);		
		
		
		int skip=0;
		Throwable t=aSRE.getCause();
		
		while (t!=null && !(t instanceof RequestFrameException || t instanceof RequestFrameError))
		{	writeFrame(t, aWriter, d);
			skip++;
			t=t.getCause();
		}
		
		return skip;
	}
	
	public static void writeTraceDebug(Writer aWriter, TraceDebugCapture aTrace, int aIndent) throws IOException
	{	int count=aTrace.getCount();
		int j=0;
		while (j<count)
		{	j=recurseWriteRFE(aWriter,aTrace,j,aIndent);
		}	
	}
	
	private static int recurseWriteRFE(Writer aWriter, TraceDebugCapture aTDC, int aIndex, int aIndent) throws IOException
	{
		int count=aTDC.getCount();
		int depth=aTDC.getDepth(aIndex);
		if (aIndent>=0)
		{	XMLUtils.writeIndent(aWriter, aIndent);
			aIndent++;
		}
		XMLUtils.writeOpenTag(aWriter,"n",-1);
		aWriter.write(XMLUtils.escape(aTDC.getDebug(aIndex).trim()));
		
		int j=aIndex+1;
		boolean recurse=false;
		while (j<count && aTDC.getDepth(j)>depth)
		{	if (!recurse)
			{	aWriter.write('\n');
				recurse=true;
			}
			j=recurseWriteRFE(aWriter,aTDC,j,aIndent);
		}
		if (aIndent>=0)
		{	if (recurse)
			{	XMLUtils.writeCloseTag(aWriter, "n", aIndent-1);
			}
			else
			{	XMLUtils.writeCloseTag(aWriter,"n",-1);
				aWriter.write('\n');
			}
		}
		else
		{	XMLUtils.writeCloseTag(aWriter,"n",-1);
		}
		
		return j;
	}
	
	private static Map sInstances = new WeakHashMap();
	private static int mLastCount;
	
	public static void countInstances(Object aInstance)
	{	synchronized(sInstances)
		{	sInstances.put(aInstance,Boolean.TRUE);
		}
		int size=sInstances.size();
		if (size<mLastCount)
		{	System.out.println("instances="+size);
		}
		mLastCount=size;
	}	
}